﻿---------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("Assessment")
local SML = AceLibrary("SharedMedia-1.0")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["History"] = true,
	["Number of fights to keep detailed information for. Use 0 to keep all."] = true,
	["Pause"] = true,
	["Pause gathering combat data."] = true,
	["Reset"] = true,
	["Click to reset fight data."] = true,
	["Data Options"] = true,
	["Data options."] = true,

    ["Enumerate Data"] = true,
    ["FilterAll"] = "Show All",
	["FilterNonZero"] = "Show Non-Zero Values",
	
	["Fight data has been reset"] = true,
} end)

-----------------------------------------------------------------------------

Assessment = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceModuleCore-2.0")
Assessment.Debug = (UnitName("player") == "Roartindon") and function(_, text) ChatFrame3:AddMessage(text) end or function() end
Assessment:RegisterDB("AssessmentDB", "AssessmentDBPerCharacter")
Assessment:RegisterDefaults('char', { fightData = { } } )
Assessment:RegisterDefaults('profile', { 
	history = 3,
	pause = false,
} )

-----------------------------------------------------------------------------

Assessment.Options = 
{
	handler = Assessment,
	type = 'group',
	args = 
	{
		reset =
		{
			type = "execute",
			name = L["Reset"],
			desc = L["Click to reset fight data."],
			func = function() Assessment:ResetFightData() end,
			order = 1,
		},
		history =
		{
			type = "range",
			name = L["History"],
			desc = L["Number of fights to keep detailed information for. Use 0 to keep all."],
			max = 20,
			min = 0,
			step = 1,
			get = function() return Assessment.db.profile.history end,
			set = function(value) Assessment.db.profile.history = value end,
			order = 10,
		},
		pause =
		{
			type = "toggle",
			name = L["Pause"],
			desc = L["Pause gathering combat data."],
			get = function() return Assessment.db.profile.pause end,
			set = function(value) Assessment.db.profile.pause  = value end,
			order = 10,
		},
		dataSpacer = 
		{
			type = 'header',
			order = 90,
		},
		data =
		{
			type = 'group',
			name = L["Data Options"],
			desc = L["Data options."],
			disabled = function() return not next(Assessment.Options.args.data.args) end,
			args = { },
			order = 100,
		}
	},
}

Assessment:RegisterChatCommand({"/as", "/assessment"}, Assessment.Options)

-----------------------------------------------------------------------------

local fightData
Assessment.ColorType			= { }
Assessment.DefaultDisplaySets	= { }
Assessment.DefaultTags			= { }
Assessment.FilterType			= { }
Assessment.ListType				= { }

-----------------------------------------------------------------------------

Assessment.FilterType.All =
{
	tag			= "1",
	description	= L["FilterAll"],
}

Assessment.FilterType.NonZero =
{
	tag			= "barValue ~= 0",
	description	= L["FilterNonZero"],
}

Assessment.ListType.EnumerateTable =
{
	method = function(data, ...)
				for i=1,select('#',...) do
					if not data then return function() return nil end end
					data = data[select(i, ...)]
				end
				if not data then return function() return nil end end
				return pairs(data)
			 end,

	data = function(listTypeData)
				if not listTypeData then return end
				return Assessment.ListType.EnumerateTable.updateData[listTypeData]
		   end,
	
	description = L["Enumerate Data"],
	
	updateData = { }
}

-----------------------------------------------------------------------------

function Assessment:OnInitialize()
	fightData = self.db.char.fightData
	
	if not fightData[0] then
		fightData[0] = self:NewFightDataTable()
	end
end

function Assessment:OnEnable()
end

function Assessment:OnDisable()
end

-----------------------------------------------------------------------------

function Assessment:ResetFightData()
	for i=0,#fightData do
		fightData[i] = self:ReleaseTable(fightData[i], true)
	end
	fightData[0] = self:NewFightDataTable()
	self:TriggerEvent("Assessment:ResetFightData")
	
	self:Print(L["Fight data has been reset"])
end

function Assessment:NewFightDataTable()
	local data = self:NewTable()
	data.dataUpdateTimes = self:NewTable()
	self:TriggerEvent("Assessment:NewFightDataTable", data)
	return data
end

-----------------------------------------------------------------------------

function Assessment:StartNewFight()
	if self.db.profile.pause then return end

	table.insert(fightData, 1, self:NewFightDataTable())
	
	if self.db.profile.history ~= 0 then
		for i=#fightData, self.db.profile.history+1, -1 do
			fightData[i] = self:ReleaseTable(fightData[i], true)
		end
	end
end

-----------------------------------------------------------------------------

local dataTablePool = setmetatable({ }, { __mode="k" })

function Assessment:NewTable()
	local recycledTable = next(dataTablePool)
	if recycledTable then
		dataTablePool[recycledTable] = nil
		return recycledTable
	else
		return { }
	end
end

function Assessment:ReleaseTable(t, recursive)
	for k,v in pairs(t) do
		if recursive and type(v) == "table" then
			self:ReleaseTable(v, true)
		end
		t[k] = nil
	end
	dataTablePool[t] = true
end

-----------------------------------------------------------------------------

function Assessment:GetFightData(index)
	if not fightData then return nil end
	return fightData[index]
end

-----------------------------------------------------------------------------

local IterateUpdateDataTables
do
	local index

	local function DataTableIterator()
		if index < 1 then
			index = index+1
			return fightData[index]
		end
	end

	function IterateUpdateDataTables()
		if Assessment.db.profile.pause then
			return function() return nil end
		else
			index = -1
			return DataTableIterator
		end
	end
end

-----------------------------------------------------------------------------

local function get(data, lookup)
	local v
	while lookup do
		v, lookup = ("."):split(lookup, 2)
		data = data[v]
	end
	return data
end

local function set(data, lookup, value)
	local v
	while true do
		v, lookup = ("."):split(lookup, 2)		
		if lookup then
			data = data[v]
		else
			data[v] = value
			return
		end
	end
end

function Assessment:RedrawAll()
	self:TriggerEvent("Assessment:RedrawAll")
end

-----------------------------------------------------------------------------

Assessment.modulePrototype.IterateUpdateDataTables = IterateUpdateDataTables
Assessment.modulePrototype.NewTable = Assessment.NewTable
Assessment.modulePrototype.ReleaseTable = Assessment.ReleaseTable
function Assessment.modulePrototype.GetOptionValue(data, lookup)
	local v
	while lookup do
		v, lookup = ("."):split(lookup, 2)
		data = data[v]
	end
	return data
end

function Assessment.modulePrototype.SetOptionValue(data, lookup, value)
	local v
	while true do
		v, lookup = ("."):split(lookup, 2)		
		if lookup then
			data = data[v]
		else
			data[v] = value
			return
		end
	end
end

function Assessment.modulePrototype:CopyOptionsTemplate(source, i)
	local result = Assessment:NewTable()

	for k,v in pairs(source) do
		if type(v) == "table" then
			result[k] = self:CopyOptionsTemplate(v, i)
		else
			if k == "passValue" then
				local passValueTable = self:NewTable()
				passValueTable[1] = i
				passValueTable[2] = v
				result[k] = passValueTable
			else
				result[k] = v
			end
		end
	end
	
	return result
end

function Assessment.modulePrototype:CopyTable(source)
	local result = self:NewTable()

	for k,v in pairs(source) do
		if type(v) == "table" then
			result[k] = self:CopyTable(v)
		else
			result[k] = v
		end
	end
	
	return result
end

function Assessment.modulePrototype:Debug(message)
	Assessment:Debug(message)
end

function Assessment.modulePrototype:CancelEvent(eventName)
	if self:IsEventRegistered(eventName) then
		self:UnregisterEvent(eventName)
	end
end

-----------------------------------------------------------------------------
